@@ -66,17 +66,28 @@ import sys
 import os
 import glob
 import imp
+import time
+import errno
 
-from StringIO import StringIO
+if sys.version_info[0] >= 3:
+    from io import StringIO
+    def reraise(tp, value, tb=None):
+        value = tp(value)
+        if value.__traceback__ is not tb:
+            raise value.with_traceback(tb)
+        raise value
+else:
+    from StringIO import StringIO
+    exec("def reraise(tp, value, tb=None):\n    raise tp, value, tb",
+         globals())
 
 from numpy.distutils.core import Extension, numpy_cmdclass, NumpyDistribution
-from distutils.dist import Distribution
-from distutils.errors import DistutilsArgError, DistutilsError, CCompilerError
+from distutils.errors import DistutilsArgError
 from distutils.util import grok_environment_error
 
-assert sys.hexversion >= 0x2030000, "need Python 2.3 or later"
+assert sys.hexversion >= 0x2060000, "need Python 2.6 or later"
 
-__version__ = "0.1"
+__version__ = "0.2"
 
 F_EXT = ".f"
 F90_EXT = ".f90"
@@ -115,6 +126,19 @@ def f_to_dll(filename, ext = None, force_rebuild = 0,
     if not fbuild_dir:
         fbuild_dir = os.path.join(path, "_fbld")
 
+    lock_fn = os.path.join(fbuild_dir, 'lock')
+    try:
+        os.makedirs(fbuild_dir)
+    except OSError:
+        pass
+
+    lock = LockFile(lock_fn)
+    with lock:
+        return _f_to_dll(filename, ext, force_rebuild, fbuild_dir,
+                         setup_args, reload_support)
+
+def _f_to_dll(filename, ext, force_rebuild ,
+              fbuild_dir, setup_args, reload_support):
     script_args=setup_args.get("script_args",[])
     if DEBUG or "--verbose" in script_args:
         quiet = "--verbose"
@@ -186,6 +210,8 @@ def f_to_dll(filename, ext = None, force_rebuild = 0,
                                           basename + '.reload%s'%count)
                     try:
                         import shutil # late import / reload_support is: debugging
+                        try: os.unlink(r_path)
+                        except OSError: pass
                         shutil.copy2(org_path, r_path)
                         so_path = r_path
                     except IOError:
@@ -269,7 +295,7 @@ def handle_dependencies(ffilename):
     # but we know more about dependencies so force a rebuild if
     # some of the dependencies are newer than the ffile.
     if os.path.exists(dependfile):
-        with open(dependfile, 'rb') as f:
+        with open(dependfile, 'r') as f:
             depends = f.readlines()
         depends = [depend.strip() for depend in depends]
 
@@ -336,9 +362,10 @@ def load_module(name, ffilename, fbuild_dir=None):
         assert mod.__file__ == so_path, (mod.__file__, so_path)
     except Exception:
         import traceback
-        raise ImportError("Building module %s failed: %s" %
-                          (name,
-                           traceback.format_exception_only(*sys.exc_info()[:2]))), None, sys.exc_info()[2]
+        reraise(ImportError, 
+                "Building module %s failed: %s" % (
+                    name, traceback.format_exception_only(*sys.exc_info()[:2])), 
+                sys.exc_info()[2])
     return mod
 
 
@@ -397,11 +424,15 @@ class FImporter(object):
                 paths = sys.path
             join_path = os.path.join
             is_file = os.path.isfile
+            is_abs = os.path.isabs
+            abspath = os.path.abspath
             #is_dir = os.path.isdir
             sep = os.path.sep
             for path in paths:
                 if not path:
                     path = os.getcwd()
+                elif not is_abs(path):
+                    path = abspath(path)
                 if is_file(path+sep+f_module_name):
                     return FLoader(fullname, join_path(path, f_module_name),
                                      fbuild_dir=self.fbuild_dir)
@@ -473,11 +504,72 @@ def install(fimport=True, build_dir=None,
         importer = FImporter(fbuild_dir=build_dir)
         sys.meta_path.append(importer)
 
+#------------------------------------------------------------------------------
+# Lock file
+#------------------------------------------------------------------------------
+
+try:
+    import posix
+    _IS_POSIX = True
+except:
+    _IS_POSIX = False
+
+if _IS_POSIX:
+    class LockFile(object):
+        """
+        Lock file (Unix-only), implemented via symlinks.
+        """
+    
+        def __init__(self, filename, timeout=0.05):
+            self.filename = filename
+            self.timeout = timeout
+            self.fd = None
+    
+        def __enter__(self):
+            tries = 0
+            while True:
+                if tries > 0:
+                    time.sleep(self.timeout)
+                try:
+                    os.symlink(str(os.getpid()), self.filename)
+                    break
+                except OSError as err:
+                    if err.errno != errno.EEXIST:
+                        raise
+    
+                    try:
+                        pid = int(os.readlink(self.filename))
+                    except OSError as err:
+                        if err.errno == errno.ENOENT:
+                            continue
+                        raise
+    
+                    # Check if it's still alive
+                    try:
+                        os.kill(pid, 0)
+                    except OSError as err:
+                        if err.errno == errno.ESRCH:
+                            # no such process
+                            os.unlink(self.filename)
+                            continue
+                        raise
+    
+        def __exit__(self, type, value, traceback):
+            os.unlink(self.filename)
+else:
+    # Dummy lockfile, does nothing
+    class LockFile(object):
+        def __init__(self, filename, timeout=0.05):
+            pass
+        def __enter__(self):
+            pass
+        def __exit__(self, type, value, traceback):
+            pass
 
 # MAIN
 
 def show_docs():
-    print __doc__
+    print(__doc__)
 
 if __name__ == '__main__':
     show_docs()
